import { world } from "@minecraft/server";
import { getDistance, log } from "../utils";

const max_stacks = 60;

export async function spawnSnowballStack() {
    if(world.scoreboard.getObjective("value").getScore("snowball_fight_players")>0) {
        let entities = await getSpawnLocations();
        let stacks = world.getDimension("overworld").getEntities({families: ["snowball_stack"]}).length;
        if(entities.length==0) {
            log("Couldn't Spawn Snowball Stack: No Empty Locations");
            return;
        }
        if(stacks>=max_stacks) {
            log("Couldn't Spawn Snowball Stack: Maximum Reached");
            return;
        }
        entities[Math.random() * entities.length | 0].runCommandAsync(`/summon podcrash:snowball_stack ~ ~ ~ 0 0 podcrash:empty "§e[§gPunch§e]"`);
        log(`Spawned 1 Snowball Stack. max_stacks=${max_stacks},stacks=${stacks+1},locations=${entities.length}`)
        if(max_stacks>(stacks+1)) spawnSnowballStack();
    }
}


async function getSpawnLocations() {
    let entities = [];
    world.getDimension("overworld").getEntities({families: ["snowball_stack_spawn"]}).forEach(e => {
        if(world.getDimension("overworld").getEntities({location: e.location, closest: 1, maxDistance: 1, families: ["snowball_stack"]}).length==0) entities.push(e);
    });
    return entities;
}

const hitMessagesSource = ["§6Ouch! You've been hit by a snowball thrown by §g{damager}§6.",
"§6Direct hit! §g{damager}§6 got you with a snowball.",
"§6You take a snowy blow from §g{damager}§6's precise aim.",
"§6Incoming snowball from §g{damager}§6! It's a hit!",
"§6{damager}§6 scores a hit on you with a well-thrown snowball.",
"§6Snowy impact! §g{damager}§6 lands a snowball on you.",
"§6It's a hit! §g{damager}§6 nails you with a snowball.",
"§6§g{damager}§6 has a sharp aim, hitting you with their snowball.",
"§6You get tagged by §g{damager}§6's snowball. Brr!",
"§6Snowball from §g{damager}§6 connects with you!"];
const hitMessages = ["§6You hit §g{target}§6 with a perfectly aimed snowball!",
"§6A snowball thrown by you finds its mark, hitting §g{target}§6.",
"§6You score a direct hit on §g{target}§6 with a snowball!",
"§6Your snowball strikes true, hitting §g{target}§6 squarely.",
"§6You make an accurate throw and hit §g{target}§6 with a snowball.",
"§6It's a hit! You hit §g{target}§6 with a snowball.",
"§6§g{target}§6 takes cover too late as your snowball lands.",
"§6You have a sharp aim, hitting §g{target}§6 with your snowball.",
"§6Your snowball connects with §g{target}§6!",
"§6Bullseye! You land a snowball right on §g{target}§6."];

world.afterEvents.projectileHitEntity.subscribe((data) => {
    if(data.source!=null 
        && data.getEntityHit().entity.typeId=="minecraft:player"
        && data.source.typeId=="minecraft:player" 
        && data.getEntityHit().entity.hasTag("playing_snowball_fight")
        && data.source.hasTag("playing_snowball_fight")) {
            data.source.runCommand(`/scoreboard players add @s snowball_fight_hits 1`)
            data.source.runCommand(`/tellraw @s { "rawtext": [ { "text": "${hitMessages[Math.random()*hitMessages.length|0].replace("{target}", data.getEntityHit().entity.nameTag)}"}]}`);
            data.getEntityHit().entity.runCommand(`/tellraw @s { "rawtext": [ { "text": "${hitMessagesSource[Math.random()*hitMessagesSource.length|0].replace("{damager}", data.source.nameTag)}"}]}`);
            data.getEntityHit().entity.runCommand(`/damage @s 0`);
            data.getEntityHit().entity.applyKnockback(data.source.getViewDirection().x, data.source.getViewDirection().y, 0.25, 0.1);
        }
});

world.afterEvents.entityHitEntity.subscribe((data) => {
    if(data.hitEntity.typeId=="podcrash:snowball_stack" && data.damagingEntity.typeId=="minecraft:player") {
        data.hitEntity.runCommand("/event entity @s podcrash:despawn");
        data.damagingEntity.runCommand("/give @s minecraft:snowball 8");
        spawnSnowballStack();
    }
});