import { EffectType, system, world } from "@minecraft/server";
import { giveCandy } from "../candy";
import { log, runCmd } from "../utils";

const spawnpoints = ["1268.67 -20.00 427.50", "1266.05 -20.00 436.46", "1270.60 -20.00 438.58", "1260.72 -20.00 442.26", "1251.64 -20.00 443.23", "1248.70 -20.00 455.39",
"1241.30 -20.00 447.27", "1237.83 -20.00 442.43", "1245.17 -20.00 426.78", "1255.80 -20.00 428.92", "1261.54 -20.00 436.50", "1270.49 -14.00 439.09", "1269.26 -14.00 427.30", "1252.30 -14.00 423.67", "1260.19 -14.00 432.14",
"1246.54 -14.00 438.89", "1246.31 -14.00 450.60", "1255.23 -14.00 452.23", "1251.30 -14.00 440.38", "1248.98 -8.00 436.39", "1260.68 -8.00 434.18", "1254.50 -8.00 428.41"]


let initialGame = true;
let running = false;
let wave = 0;
let mobs = ["zombie"];
let mobsAlive = 0;
let mobsGoal;
let delayUntilNextWave = -1;

world.afterEvents.entityDie.subscribe((data) => {
    if (running && data.deadEntity.typeId != "minecraft:item" && data.deadEntity.hasTag("workshop_enemy")) {
        mobsAlive--;
        if (data.damageSource.damagingEntity != null && data.damageSource.damagingEntity.typeId == "minecraft:player" && (Math.random() * 3 | 0) == 0) giveCandy(data.damageSource.damagingEntity, "0-4");
    }
    if (running && data.deadEntity.typeId == "minecraft:player") {
        data.deadEntity.addTag("died_in_workshop");
        endWorkshop();
        runCmd(`/titleraw @a subtitle { "rawtext": [ { "translate": "game.workshop.defeat.subtitle" } ] }`);
        runCmd(`/titleraw @a title { "rawtext": [ { "translate": "game.workshop.defeat.title" } ] }`);
    }
});

world.afterEvents.playerSpawn.subscribe((data) => {
    if (!running && data.initialSpawn) {
        runCmd(`/execute positioned 1247 -20 442 as @a[name="${data.player.name}",r=75] run function workshop/rejoin`);
    }
    if (data.player.hasTag("died_in_workshop")) {
        data.player.teleport({ x: 673.5, y: -20.00, z: 443.5 });
        data.player.removeTag("died_in_workshop");
    }
});


system.runInterval(() => {
    if (running) {
        mobsAlive = world.getDimension("overworld").getEntities({ tags: ["workshop_enemy"] }).length;
        runCmd(`/kill @e[type=minecraft:item]`);
        runCmd(`/titleraw @a actionbar { "rawtext": [ { "text": "§pWave: §g${wave}   §pMobs Alive: §g${mobsAlive}" } ] }`);
        if (mobsAlive == 0) {
            if (delayUntilNextWave == 0) {
                delayUntilNextWave = -1;
                nextWave();
            }
            if (delayUntilNextWave > 0) {
                runCmd(`/titleraw @a actionbar { "rawtext": [ { "text": "§pLoading..." } ] }`);
                delayUntilNextWave--;
            }
        }
    } else {
        let entities = world.getDimension("overworld").getEntities({ tags: ["workshop_enemy"] });
        if (entities.length > 0) {
            entities.forEach((e) => { e.kill() });
        }
    }
});

export function equipPlayer(player) {
    const type = wave > 30 ? "netherite" : wave > 15 ? "diamond" : "iron";
    player.runCommand(`execute if entity @s[tag=editing_furniture] run function furniture/exit_editor`);
    player.runCommand(`clear @s`);
    player.runCommand(`replaceItem entity @s slot.armor.head 0 minecraft:${type}_helmet 1 0`);
    player.runCommand(`replaceItem entity @s slot.armor.chest 0 minecraft:${type}_chestplate 1 0`);
    player.runCommand(`replaceItem entity @s slot.armor.legs 0 minecraft:${type}_leggings 1 0`);
    player.runCommand(`replaceItem entity @s slot.armor.feet 0 minecraft:${type}_boots 1 0`);
    player.runCommand(`replaceItem entity @s slot.hotbar 0 minecraft:${type}_sword 1 0`);
    player.runCommand(`replaceItem entity @s slot.hotbar 1 minecraft:bow 1 0`);
    player.runCommand(`replaceItem entity @s slot.hotbar 8 minecraft:arrow 64 0`);
    player.runCommand(`replaceItem entity @s slot.weapon.offhand 0 minecraft:shield 1 0`);
}

function initialWin() {
    runCmd("/setblock 677 -22 442 redstone_torch");
    runCmd(`/tellraw @a { "rawtext": [ { "translate": "game.workshop.victory.chat" } ] }`);
    world.scoreboard.getObjective("value").setScore("santas_workshop_defeated", 1);
    runCmd(`/titleraw @a subtitle { "rawtext": [ { "translate": "game.workshop.victory.subtitle" } ] }`);
    runCmd(`/titleraw @a title { "rawtext": [ { "translate": "game.workshop.victory.title" } ] }`);
    runCmd(`/execute positioned 675 -20 441 as @e[type=podcrash:ella] run dialogue change @s ella_defeated`);
    runCmd(`/fill 705 -20 424 708 -17 424 air`);
    endWorkshop();
}

function nextWave() {
    if (initialGame && wave == 15) {
        initialWin();
        return;
    }
    delayUntilNextWave = 5;
    wave++;
    mobsGoal += 3;
    if (wave == 15) mobsGoal = 50;
    runCmd("/kill @e[type=item]");
    world.getAllPlayers().forEach((player) => {
        player.addEffect("regeneration", 60, { amplifier: 255, showParticles: true });
        equipPlayer(player);
    });
    if (wave == 3) mobs.push("skeleton");
    if (wave == 8) {
        mobs.push("zombie");
        mobs.push("spider");
    }
    if (wave == 10) {
        mobs.push("cave_spider");
        mobs.push("zombie");
    }
    if (wave == 12) mobs.push("creeper");
    if (wave == 16) {
        mobs.push("zombie");
        mobs.push("blaze");
    }
    if (wave == 20) {
        mobs.push("wither_skeleton");
    }
    if (wave == 22) {
        mobs.push("pillager");
    }
    if (wave == 25) {
        mobs.push("vindicator");
        mobs.push("zombie");
        mobs.push("skeleton");
    }
    if (wave == 40) {
        mobs.push("evocation_illager");
        mobs.push("vindicator");
    }
    if (wave > 40) {
        mobs.push("vindicator");
        mobs.push("evocation_illager");
    }
    while (mobsAlive < mobsGoal) {
        mobsAlive += 1;
        let loc = spawnpoints[Math.random() * spawnpoints.length | 0].split(" ");
        const mob = world.getDimension("overworld").spawnEntity(`minecraft:${mobs[Math.random() * mobs.length | 0]}`, { x: Number(loc[0]), y: Number(loc[1]), z: Number(loc[2]) });
        mob.addTag("workshop_enemy");
    }
    runCmd(`/titleraw @a subtitle { "rawtext": [ { "text": "§g${mobsGoal} Enemies" } ] }`);
    if (initialGame && wave == 15) runCmd(`/titleraw @a subtitle { "rawtext": [ { "text": "§cFinal Wave" } ] }`);
    runCmd(`/titleraw @a title { "rawtext": [ { "text": "§6Wave ${wave}" } ] }`);
}

export function endWorkshop() {
    running = false;
    mobsAlive = 0;
    world.getDimension("overworld").getEntities({ tags: ["workshop_enemy"] }).forEach((e) => {
        e.runCommand("/tp @s ~ -70 ~");
    });
    world.getAllPlayers().forEach((player) => {
        player.teleport({ x: 673.5, y: -20.00, z: 443.5 });
        player.removeTag("playing_workshop");
    });
}

export function startWorkshop() {
    wave = 0;
    mobs = ["zombie"];
    mobsAlive = 0;
    mobsGoal = 3;
    running = true;
    initialGame = world.scoreboard.getObjective("value").getScore("santas_workshop_defeated") == 0;
    world.getDimension("overworld").getEntities({ tags: ["workshop_enemy"] }).forEach((e) => {
        e.runCommand("/tp @s ~ -70 ~");
        e.kill();
    });
    world.getAllPlayers().forEach((player) => {
        player.addTag("playing_workshop");
        player.teleport({ x: 1247.98, y: -20.00, z: 442.15 });
    });
    delayUntilNextWave = 20;
}