import { ActionFormData } from "@minecraft/server-ui";
import { log } from "../utils";
import { world } from "@minecraft/server";

export default class Shop {
    constructor(player, title, description) {
        this.player = player;
        this.candyBoard = world.scoreboard.getObjective("candy");
        this.prices = [];
        this.items = [];
        this.onSuccess = [];
        this.form = new ActionFormData();
        this.form.title(title);
        this.form.body(description + `\n\n                §rYou have §a${this.candyBoard.getScore(player)} §r\uE10A\n\n`);
    }

    addElement(price, item, success) {
        this.form.button(`${price}x \uE10A   ${this.candyBoard.getScore(this.player)<price?"\uE109":"\uE108"}   ${item}`)
        this.prices.push(price);
        this.items.push(item);
        this.onSuccess.push(success);
    }

    open() {
        if(this.items.length==0) {
            this.form.body("\nThis Shop is currently Empty!\n\n\n\n\n\n\n\n\n\n\n\n");
            this.form.button("Exit");
            this.form.show(this.player);
            return;
        }
        this.form.show(this.player).then((response) => {
            if(response.canceled||response.selection===undefined) return;
            const balance = this.candyBoard.getScore(this.player);
            if(balance<this.prices[response.selection]) {
                this.player.runCommand(`/tellraw @s { "rawtext": [ { "translate": "shop.not_enough_candy" }, { "text": "${this.items[response.selection]}"} ] }`);
            } else {
                this.player.runCommand(`/tellraw @s { "rawtext": [ { "translate": "shop.purchase" }, { "text": "${this.items[response.selection]}"} ] }`);
                this.candyBoard.setScore(this.player, balance-this.prices[response.selection]);
                this.onSuccess[response.selection]();
            }
        });
    }


}