import {world, BlockInventoryComponent, ItemStack} from "@minecraft/server";

const MAX_CHARS = 27 * 50 * 20

export default class JSONChest {
    constructor(location = {x: 0, y: -64, z: 0}, size = 1, dimensionId = "overworld") {
        this.containers = []
        this.dimension = world.getDimension(dimensionId)

        try {
            for (let i = 0; i < size; i++) {
                this.dimension.runCommand(
        `execute positioned ${location.x} ${location.y} ${location.z} unless block ~ ~ ~ chest run setblock ~ ~ ~ chest`
                )

                this.containers.push(this.dimension
                    .getBlock(location)
                    .getComponent(BlockInventoryComponent.componentId)["container"])

                location.y++
            }
        } catch (e) {
            throw new Error("Ensure valid JSONChest location within chunks that are always loaded.")
        }

    }

    set(object) {
        const stringified = JSON.stringify(object) ?? ""

        if (stringified.length > MAX_CHARS * this.containers.length)
            throw new Error(`Object too large to store in JSONChest (exceeds ${MAX_CHARS} char per chest limit). JSONChest size must be ${Math.ceil(stringified.length/MAX_CHARS)} or greater to accommodate this object.`)

        const lore_array = []
        stringified.match(/.{1,1000}/g)?.forEach((d, i) => {
            lore_array[i] = d.match(/.{1,50}/g)
        })

        const item = new ItemStack("minecraft:feather", 1)

        this.containers.forEach((container, index) => {
            container.clearAll()

            for (let i = index * container.size; i < (index + 1) * container.size; i++) {
                if (lore_array[i] === undefined) break

                item.setLore(lore_array[i])
                container.setItem(i - index * container.size, item)
            }
        })
    }

    get() {
        let stringified = ""

        this.containers.forEach(container => {
            for (let i = 0; i < container.size; i++) {
                const lore = container.getItem(i)?.getLore()

                if (lore === undefined) break

                stringified += lore.join("")
            }
        })

        if (stringified.length === 0) return undefined

        try {
            return JSON.parse(stringified)
        } catch (e) {
            return undefined
        }
    }
}