import { BlockInventoryComponent, EntityInventoryComponent, world } from "@minecraft/server";
import JSONChest from "./JSONChest";
import { log } from "../utils";

const overworld = world.getDimension("overworld");

export function save(player) {
    try {
        const jsonChest = new JSONChest();
        const playerList = jsonChest.get() ?? [];
        let y = -64 + playerList.length;
        if (playerList.includes(player.id)) {
            y = -64 + playerList.indexOf(player.id);
        } else {
            playerList.push(player.id);
            jsonChest.set(playerList);
        }
        const playerInventory = player.getComponent(EntityInventoryComponent.componentId).container;
        overworld.runCommand(`execute positioned 2 ${y} 0 unless block ~ ~ ~ chest run setblock ~ ~ ~ chest`);
        overworld.runCommand(`execute positioned 3 ${y} 0 unless block ~ ~ ~ chest run setblock ~ ~ ~ chest`);
        const chestInventory = world.getDimension("overworld").getBlock({ x: 2, y: y, z: 0 }).getComponent(BlockInventoryComponent.componentId).container;
        if (chestInventory.size < playerInventory.size) throw new Error("chest inventory too small to store player inventory");
        for (let i = 0; i < playerInventory.size; i++) {
            chestInventory.setItem(i, playerInventory.getItem(i));
        }
    } catch (e) {
        log(e);
    }
}

export function load(player) {
    try {
        const jsonChest = new JSONChest();
        const playerList = jsonChest.get() ?? [];
        player.runCommand("/clear @s");
        if (playerList.includes(player.id)) {
            let y = -64 + playerList.indexOf(player.id);
            const playerInventory = player.getComponent(EntityInventoryComponent.componentId).container;
            if (!overworld.getBlock({ x: 2, y: y, z: 0 }).permutation.matches("minecraft:chest") || !overworld.getBlock({ x: 2, y: y, z: 0 }).permutation.matches("minecraft:chest")) {
                overworld.runCommand(`execute positioned 2 ${y} 0 unless block ~ ~ ~ chest run setblock ~ ~ ~ chest`);
                overworld.runCommand(`execute positioned 3 ${y} 0 unless block ~ ~ ~ chest run setblock ~ ~ ~ chest`);
                addDefaults(player);
                throw new Error("player registered, but chest not found");
            }
            const chestInventory = world.getDimension("overworld").getBlock({ x: 2, y: y, z: 0 }).getComponent(BlockInventoryComponent.componentId).container;
            if (chestInventory.size < playerInventory.size) throw new Error("chest inventory too small to store player inventory");
            for (let i = 0; i < playerInventory.size; i++) {
                playerInventory.setItem(i, chestInventory.getItem(i));
            }
        } else {
            addDefaults(player);
        }
    } catch (e) {
        log(e);
    }
}

function addDefaults(player) {
    player.runCommand("/replaceitem entity @s slot.hotbar 8 podcrash:exit_furniture");
    player.runCommand("/replaceitem entity @s slot.hotbar 7 podcrash:furniture_tool");

}