import { BlockInventoryComponent, EntityInventoryComponent, world } from "@minecraft/server";

const overworld = world.getDimension("overworld");

export function getNearestPlayer(loc) {
    return getPlayersSortedByDistance(loc)[0];
}

export function getPlayersSortedByDistance(loc) {
    return world.getPlayers().sort((a, b) => getDistance(a.location, loc) - getDistance(b.location, loc));
}

export function dropItems(p, typeId) {
    const player = world.getDimension("overworld").getPlayers({ name: p.name })[0];
    const loc = player.location;
    loc.y = loc.y + 4;
    try {
        player.runCommand(`setblock ${loc.x} ${loc.y} ${loc.z} hopper`);
        const playerInventory = player.getComponent(EntityInventoryComponent.componentId).container;
        const hopperInventory = world.getDimension("overworld").getBlock(loc).getComponent(BlockInventoryComponent.componentId).container;
        let hopperSlot = 0;
        for (let i = 0; i < playerInventory.size; i++) {
            if (hopperSlot == 5) break;
            const item = playerInventory.getItem(i);
            if (item === undefined || item.typeId === undefined) continue;
            if (item.typeId === typeId) {
                hopperInventory.setItem(hopperSlot, item);
                hopperSlot++;
            }
        }
        player.runCommand(`clear @s ${typeId}`)
        player.runCommand(`setblock ${loc.x} ${loc.y} ${loc.z} air destroy`);
        player.runCommand(`kill @e[type=minecraft:item,name="Hopper"]`);
        player.runCommand(`titleraw @s actionbar { "rawtext": [ { "translate": "feature.furniture.inventory_warn" } ] }`);
    } catch (e) {
        log(e);
        player.runCommand(`setblock ${loc.x} ${loc.y} ${loc.z} air destroy`);
        player.runCommand(`kill @e[type=minecraft:item,name="Hopper"]`);
    }
}

export function getDistance(a, b) {
    let x = a.x - b.x;
    let y = a.y - b.y;
    let z = a.z - b.z;
    return Math.sqrt(x * x + y * y + z * z);
}

export async function log(...args) {
    await runCmd(
        `tellraw @a[tag=receive_logs] {"rawtext":[{"text":"${args
            .map((d) => process(d))
            .join(" ")}"}]}`
    );
}

export const runCmd = async (commandString) =>
    await world.getDimension("overworld").runCommandAsync(commandString);

function process(arg) {
    let output = arg?.toString();

    if (Array.isArray(arg)) output = `[${arg.map((d) => process(d)).join(", ")}]`;
    else if (arg === null) output = `§6null`;
    else
        switch (typeof arg) {
            case "string":
                output = `§a${arg.startsWith("Error") ? "" : '"'}§2${arg}§a${arg.startsWith("Error") ? "" : '"'
                    }`;
                break;
            case "number":
                output = isNaN(arg) ? `§5§oNaN` : `§9${arg}`;
                break;
            case "boolean":
                output = `§6${output}`;
                break;
            case "undefined":
                output = `§6undefined`;
                break;
        }

    return `${output.replaceAll('"', '\\"')}§r`;
}